@@ -304,21 +304,30 @@ class STT(object):
     #####################################################################################
     
     def __add__(A,B):
-        if not (A.init and B.init): raise RuntimeError("The STT are not initialized")
-        
+        # if not (A.init and B.init): raise RuntimeError("The STT are not initialized")
         C = A.copy()
         C += B
         return C
     
     def __iadd__(A,B):
-        if not (A.init and B.init): raise RuntimeError("The STT are not initialized")
-        
-        for (point,TTa),(_,TTb) in \
-                zip( np.ndenumerate(A.TTapprox), np.ndenumerate(B.TTapprox) ):
-            A.TTapprox[point] = (TTa + TTb).rounding( min(A.eps,B.eps) )
-        
-        A.prepare_surrogate(force_redo=True)
-        
+        if not ( (isinstance(A, STT) and A.init) and \
+                     ((isinstance(B,STT) and B.init) or isinstance(B, np.ndarray) or isinstance(B, float))): 
+            raise RuntimeError("The STT are not initialized")
+        if isinstance(A, STT) and isinstance(B, STT):
+            for (point,TTa),(_,TTb) in \
+                    zip( np.ndenumerate(A.TTapprox), np.ndenumerate(B.TTapprox) ):
+                A.TTapprox[point] = (TTa + TTb).rounding( min(A.eps,B.eps) )
+
+        elif isinstance(A, STT) and isinstance(B, np.ndarray):
+            for (point,TTa),(_,b) in \
+                    zip( np.ndenumerate(A.TTapprox), np.ndenumerate(B) ):
+                A.TTapprox[point] = (TTa + b).rounding( A.eps )
+
+        elif isinstance(A, STT) and isinstance(B, float):
+            for (point,TTa) in np.ndenumerate(A.TTapprox):
+                A.TTapprox[point] = (TTa + B).rounding( A.eps )
+
+        A.prepare_surrogate(force_redo=True)        
         return A
     
     def __neg__(A):
@@ -327,39 +336,44 @@ class STT(object):
         return B
 
     def __sub__(A,B):
-        if not (A.init and B.init): raise RuntimeError("The STT are not initialized")
+        # if not (A.init and B.init): raise RuntimeError("The STT are not initialized")
         return A + (-B)
     
     def __isub__(A,B):
-        if not (A.init and B.init): raise RuntimeError("The STT are not initialized")
+        # if not (A.init and B.init): raise RuntimeError("The STT are not initialized")
         A += -B
         return A
     
     def __mul__(A,B):
-        if not (A.init and B.init): raise RuntimeError("The STT are not initialized")
-        
+        # if not (A.init and B.init): raise RuntimeError("The STT are not initialized")
         C = A.copy()
         C *= B
         return C
     
     def __rmul__(A,B):
-        if not (A.init and B.init): raise RuntimeError("The STT are not initialized")
+        # if not (A.init and B.init): raise RuntimeError("The STT are not initialized")
         return (A * B)
     
     def __imul__(A,B):
+        if not ( (isinstance(A, STT) and A.init) and \
+                     ((isinstance(B,STT) and B.init) or isinstance(B, np.ndarray) or isinstance(B, float))): 
+            raise RuntimeError("The STT are not initialized")
         if isinstance(A,STT) and isinstance(B,STT):
-            if not (A.init and B.init): raise RuntimeError("The STT are not initialized")
             for (point,TTa),(_,TTb) in \
                 zip( np.ndenumerate(A.TTapprox), np.ndenumerate(B.TTapprox) ):
                 A.TTapprox[point] = (TTa * TTb).rounding( min(A.eps,B.eps) )
-            A.prepare_surrogate(force_redo=True)
-            return A
+        
         elif isinstance(B,float) and isinstance(A,STT):
-            if not (A.init): raise RuntimeError("The STT is not initialized")
             for (point,TTa) in np.ndenumerate(A.TTapprox):
                 A.TTapprox[point] = (TTa * B).rounding( min(A.eps) )
-            A.prepare_surrogate(force_redo=True)
-            return A
+
+        elif isinstance(A, STT) and isinstance(B, np.ndarray):
+            for (point,TTa),(_,b) in \
+                    zip( np.ndenumerate(A.TTapprox), np.ndenumerate(B) ):
+                A.TTapprox[point] = (TTa * b).rounding( A.eps )
+
+        A.prepare_surrogate(force_redo=True)
+        return A
     
     def __pow__(A,n):
         if not A.init: raise RuntimeError("The STT is not initialized")
