@@ -28,6 +28,7 @@ import sys
 import warnings
 import progressbar
 import itertools
+import copy
 
 import numpy as np
 import numpy.random as npr
@@ -38,7 +39,8 @@ from scipy import sparse as scsp
 from scipy.sparse import linalg as spla
 
 from TensorToolbox.core import ConvergenceError,TTcrossLoopError,\
-    TensorWrapper,maxvol,idxfold,idxunfold,ttcross_store, object_store
+    TensorWrapper,maxvol,idxfold,idxunfold,ttcross_store, object_store,\
+    Candecomp
 from TensorToolbox import multilinalg as mla
 
 class TTvec(object):
@@ -78,19 +80,23 @@ class TTvec(object):
     ttcross_Jinit = None
     ltor_fiber_lists = None
     rtol_fiber_lists = None
+
+    serialize_list = ['TT', 'init', 'method', 'eps', 'ttcross_rs', 'ttcross_Js', 'ttcross_Is', 'ttcross_Js_last', 'ttcross_Jinit', 'ltor_fiber_lists', 'rtol_fiber_lists']
+
+    subserialize_list = ['A']   # Not serialized thanks to the STT interface
     
     def __init__(self,A, eps=1e-10, method='svd', lr_r=None, lr_fix_rank=False, lr_Jinit=None, lr_delta=1e-4, lr_maxit=100, mv_eps=1e-6,mv_maxit=100,lr_store_location="",store_object=None,store_freq=1, generate=True):
 
-        from candecomp import Candecomp
         # Initialize the tensor with the input tensor in TT ([][]numpy.ndarray), tensor(numpy.ndarray) or CANDECOMP form
         
         self.A = A
         
         if generate:
             if isinstance(A,Candecomp):
+                self.eps = eps
                 self.TT = A.to_TT()
                 self.init = True
-                self.rounding(eps)
+                self.rounding(self.eps)
             elif isinstance(A,np.ndarray) or isinstance(A,TensorWrapper):
                 self.eps = eps
                 if method == 'svd' or method == None: 
@@ -108,6 +114,14 @@ class TTvec(object):
             else:
                 raise NameError("TensorToolbox.TTvec.__init__: Input type not allowed")
 
+    def __getstate__(self):
+        dd = dict( [ (tag, getattr( self, tag )) for tag in self.serialize_list ] )
+        return dd
+    
+    def __setstate__(self,state,f = None):
+        for tag in state.keys():
+            setattr(self, tag, state[tag])
+        
     def __getitem__(self,idxs):
         if not self.init: raise NameError("TensorToolbox.TTvec.__getitem__: TT not initialized correctly")
         # TT format: TT[i][idx] = core (matrix or row/col vector)
@@ -168,17 +182,13 @@ class TTvec(object):
         return np.array(idxs)
 
     def copy(self):
-        newTT = []
-        for TTi in self.TT: newTT.append(TTi.copy())
-        return TTvec(newTT)
+        return copy.deepcopy(self)
 
     ###########################################
     # Multi-linear Algebra
     ###########################################
 
     def __add__(A,B):
-        if not A.init or not B.init: raise NameError("TensorToolbox.multilinalg.add: TT not initialized correctly")
-        
         C = A.copy()
         C += B
         return C
@@ -187,8 +197,18 @@ class TTvec(object):
         """
         In place addition
         """
-        if not A.init or not B.init: raise NameError("TensorToolbox.multilinalg.add: TT not initialized correctly")
+        if not ( (isinstance(A, TTvec) and A.init) and \
+                     ((isinstance(B,TTvec) and B.init) or isinstance(B, float)) ): 
+            raise NameError("TensorToolbox.multilinalg.add: TT not initialized correctly")
 
+        if isinstance(A,TTvec) and isinstance(B,float):
+            lscp = [ np.ones((1,sh)) for i,sh in enumerate(A.shape()) ]
+            lscp[0] *= B
+            CP = Candecomp( lscp )
+            B = TTvec(CP,A.eps)
+            if not B.init:
+                raise NameError("TensorToolbox.multilinalg.add: TT not initialized correctly")
+            
         for i in range(len(A.TT)):
             if i == 0:
                 A.TT[i] = np.concatenate((A.TT[i],B.TT[i]),axis=2)
